Imports System.Data
Imports System.Data.SqlClient
Imports System.Text
Imports System.Security.Cryptography
Imports System.IO
Imports System.Text.RegularExpressions

Public Class FormsAuthHelper
    Public Shared Function IsUser(ByVal email As String) As Boolean
        ' real apps: implement credential validation
        ' using e.g. a hashed lookup against a directory or DB or AD
        If email <> String.Empty Then
            IsUser = True
        Else
            IsUser = False
        End If
    End Function

    Public Shared Function IsAdmin(ByVal email As String) As Boolean
        ' real apps: implement credential validation
        ' using e.g. a hashed lookup against a directory or DB or AD
        If email = "jdoe@somewhere.com" Then
            IsAdmin = True
        Else
            IsAdmin = False
        End If
    End Function

    Public Shared Function ArraysEqual(ByVal array1 As Byte(), ByVal array2 As Byte()) As Boolean
        Dim result As Boolean = False
        ' validate arrays
        If IsNothing(array1) Then
            Throw New ArgumentNullException("array1")
        End If

        If IsNothing(array2) Then
            Throw New ArgumentNullException("array2")
        End If

        ' check one: if lengths are unequal, bail
        Dim foundMismatch As Boolean = False
        Dim i As Int32
        If array1.Length = array2.Length Then
            For i = 0 To array1.Length - 1
                If array1(i) <> array2(i) Then
                    foundMismatch = True
                    Exit For
                End If
            Next i

            If Not foundMismatch Then
                result = True
            End If
        End If

        ArraysEqual = result
    End Function


    Public Shared Function AuthenticateUser(ByVal acct As String, ByVal passwd As String) As FormsUserType
        Dim conn As SqlConnection = Nothing
        Dim dsn As String = ConfigurationSettings.AppSettings("dsn")
        Dim hash As String
        Dim salt As String

        ' first, validate user name.  If that fails, return unauthenticated
        ' we're already validating it on the login form, but just to be extra
        ' paranoid in the business layer, we'll revalidate here
        ' since the password is never stored, echoed back, or used in SQL (we just hash it), 
        ' we're less concerned about what it contains. 
        Dim userRegex As Regex = New Regex("^\w+([-+.]\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*$")
        Dim m As Match = userRegex.Match(acct)
        If m.Success Then
            Try
                conn = New SqlConnection(dsn)
                conn.Open()

                Dim proc As SqlCommand

                ' get the password hash and the salt for this user account
                proc = New SqlCommand("sp_getuserdetails", conn)
                proc.CommandType = CommandType.StoredProcedure

                Dim parm As SqlParameter
                parm = proc.Parameters.Add("@acctname", SqlDbType.VarChar, 64)
                parm.Direction = ParameterDirection.Input
                parm.Value = acct

                ' the stored password hash and the salt are
                ' saved in the DB and returned as output parms
                ' the password itself is never stored (just the verifier)
                parm = proc.Parameters.Add("@passhash", SqlDbType.VarChar, 50)
                parm.Direction = ParameterDirection.Output

                parm = proc.Parameters.Add("@passsalt", SqlDbType.VarChar, 50)
                parm.Direction = ParameterDirection.Output

                proc.ExecuteNonQuery()
                hash = CType(proc.Parameters("@passhash").Value, String)
                salt = CType(proc.Parameters("@passsalt").Value, String)

                ' did we fail to get a hash or salt back?  If so, then reject 
                If Nothing = hash Or Nothing = salt Then
                    AuthenticateUser = FormsUserType.UnAuthenticated
                Else
                    Dim saltBits() As Byte = Convert.FromBase64String(salt)
                    Dim hashBits() As Byte = Convert.FromBase64String(hash)
                    Dim passBits() As Byte = Encoding.Unicode.GetBytes(passwd)

                    ' hash the password we got back with the salt we stored
                    Dim hashAlg As HashAlgorithm = SHA1.Create()
                    Dim cs As CryptoStream = New CryptoStream(Stream.Null, hashAlg, CryptoStreamMode.Write)
                    cs.Write(passBits, 0, passBits.Length)
                    cs.Write(saltBits, 0, saltBits.Length)
                    cs.FlushFinalBlock()
                    cs.Close()

                    ' now compare our new hash with the stored hash
                    ' if they match, the user is valid
                    Dim digest() As Byte = hashAlg.Hash
                    If ArraysEqual(digest, hashBits) Then
                        If IsAdmin(acct) Then
                            AuthenticateUser = FormsUserType.Admin
                        Else
                            AuthenticateUser = FormsUserType.User
                        End If
                    Else
                        AuthenticateUser = FormsUserType.UnAuthenticated
                    End If               

                End If


            Catch ex As Exception
                AuthenticateUser = FormsUserType.UnAuthenticated
            Finally
                If Not (IsNothing(conn)) Then
                    conn.Close()
                End If
            End Try
        Else
            AuthenticateUser = FormsUserType.UnAuthenticated
        End If
    End Function


    Public Shared Function GetRoleString(ByVal userType As FormsUserType)
        If userType = FormsUserType.Admin Then
            GetRoleString = "Admin"
        ElseIf userType = FormsUserType.User Then
            GetRoleString = "User"
        Else
            GetRoleString = ""
        End If
    End Function
End Class
