Imports System
Imports System.Runtime.InteropServices
Imports System.Text
Imports Microsoft.VisualBasic

Public NotInheritable Class DataProtection

    ' takes a string and returns a base64 encoded string containing
    ' the protected data
    Public Overloads Shared Function ProtectData(ByVal data As String, ByVal name As String, ByVal optEntropy() As Byte) As String
        If IsNothing(data) Then
            Throw New ArgumentNullException("data")
        End If
        If Nothing = name Then
            Throw New ArgumentNullException("name")
        End If
        Return ProtectData(data, name, NativeMethods.UIForbidden Or NativeMethods.LocalMachine, optEntropy)
    End Function 'ProtectData


    Public Overloads Shared Function ProtectData(ByVal data As String, ByVal name As String, ByVal flags As Integer, ByVal optEntropy() As Byte) As String
        If IsNothing(data) Then
            Throw New ArgumentNullException("data")
        End If
        If IsNothing(name) Then
            Throw New ArgumentNullException("name")
        End If
        Dim dataIn As Byte() = Encoding.Unicode.GetBytes(data)
        Dim dataOut As Byte() = ProtectData(dataIn, name, flags, optEntropy)
        Dim strOut As String = Nothing
        If Not IsNothing(dataOut) Then
            strOut = Convert.ToBase64String(dataOut)
        End If
        Return strOut
    End Function 'ProtectData


    ' takes a base64 encoded containing protected data and
    ' returns the data as a string
    Public Overloads Shared Function UnprotectData(ByVal data As String, ByVal optEntropy() As Byte) As String
        If IsNothing(data) Then
            Throw New ArgumentNullException("data")
        End If

        Dim dataIn As Byte() = Convert.FromBase64String(data)
        Dim dataOut As Byte() = UnprotectData(dataIn, optEntropy)
        Dim strOut As String = Nothing

        If Not IsNothing(dataOut) Then
            strOut = Encoding.Unicode.GetString(dataOut)
        End If
        Return strOut
    End Function 'UnprotectData


    Public Overloads Shared Function ProtectData(ByVal data() As Byte, ByVal name As String, ByVal optEntropy() As Byte) As Byte()
        If IsNothing(data) Then
            Throw New ArgumentNullException("data")
        End If
        If IsNothing(name) Then
            Throw New ArgumentNullException("name")
        End If
        Return ProtectData(data, name, NativeMethods.UIForbidden Or NativeMethods.LocalMachine, optEntropy)
    End Function 'ProtectData


    Public Overloads Shared Function UnprotectData(ByVal data() As Byte, ByVal optEntropy() As Byte) As Byte()
        If IsNothing(data) Then
            Throw New ArgumentNullException("data")
        End If
        Return UnprotectData(data, NativeMethods.UIForbidden, optEntropy)
    End Function 'UnprotectData


    Private Overloads Shared Function UnprotectData(ByVal data() As Byte, ByVal dwFlags As Integer, ByVal optEntropy() As Byte) As Byte()
        Dim clearText As Byte() = Nothing

        If IsNothing(data) Then
            Throw New ArgumentNullException("data")
        End If

        ' copy data into unmanaged memory
        Dim din As New NativeMethods.DATA_BLOB()
        din.cbData = data.Length
        din.pbData = Marshal.AllocHGlobal(din.cbData)
        Marshal.Copy(data, 0, din.pbData, din.cbData)

        ' using optional entropy helps a little if someone
        ' just tries to walk through the registry and decrypt
        ' secrets that look like machine encrypted DPAPI blobs
        ' you need to have the same entropy for decryption
        Dim optEntBlob As New NativeMethods.DATA_BLOB()
        optEntBlob.cbData = optEntropy.Length
        optEntBlob.pbData = Marshal.AllocHGlobal(optEntBlob.cbData)
        Marshal.Copy(optEntropy, 0, optEntBlob.pbData, optEntBlob.cbData)

        Dim dout As New NativeMethods.DATA_BLOB()

        Try
            Dim ret As Boolean = NativeMethods.CryptUnprotectData(din, Nothing, optEntBlob, IntPtr.Zero, IntPtr.Zero, dwFlags, dout)

            If ret Then
                clearText = New Byte(dout.cbData) {}
                Marshal.Copy(dout.pbData, clearText, 0, dout.cbData)
                NativeMethods.LocalFree(dout.pbData)
            Else
                Throw New Exception("Decryption failed: " + Marshal.GetLastWin32Error().ToString())
            End If

        Finally
            If Not IntPtr.Zero.Equals(din.pbData) Then
                Marshal.FreeHGlobal(din.pbData)
            End If
            If Not IntPtr.Zero.Equals(optEntBlob.pbData) Then
                Marshal.FreeHGlobal(optEntBlob.pbData)
            End If
        End Try
        Return clearText
    End Function 'UnprotectData


    Private Overloads Shared Function ProtectData(ByVal data() As Byte, ByVal name As String, ByVal dwFlags As Integer, ByVal optEntropy() As Byte) As Byte()
        Dim cipherText As Byte() = Nothing

        If IsNothing(data) Then
            Throw New ArgumentNullException("data")
        End If
        If IsNothing(name) Then
            Throw New ArgumentNullException("name")
        End If

        ' copy data into unmanaged memory
        Dim din As New NativeMethods.DATA_BLOB()
        din.cbData = data.Length
        din.pbData = Marshal.AllocHGlobal(din.cbData)
        Marshal.Copy(data, 0, din.pbData, din.cbData)

        ' using optional entropy helps a little if someone
        ' just tries to walk through the registry and decrypt
        ' secrets that look like machine encrypted DPAPI blobs
        ' you need to have the same entropy for decryption
        Dim optEntBlob As New NativeMethods.DATA_BLOB()
        optEntBlob.cbData = optEntropy.Length
        optEntBlob.pbData = Marshal.AllocHGlobal(optEntBlob.cbData)
        Marshal.Copy(optEntropy, 0, optEntBlob.pbData, optEntBlob.cbData)

        Dim dout As New NativeMethods.DATA_BLOB()

        Try
            Dim ret As Boolean = NativeMethods.CryptProtectData(din, _
                name, _
                optEntBlob, _
                IntPtr.Zero, _
                IntPtr.Zero, _
                dwFlags, _
                dout)
            If ret Then
                cipherText = New Byte(dout.cbData) {}
                Marshal.Copy(dout.pbData, cipherText, 0, dout.cbData)
                NativeMethods.LocalFree(dout.pbData)
            Else
                Throw New Exception("Encryption failed: " + Marshal.GetLastWin32Error().ToString())
            End If

        Finally
            If Not IntPtr.Zero.Equals(din.pbData) Then
                Marshal.FreeHGlobal(din.pbData)
            End If

            If Not IntPtr.Zero.Equals(optEntBlob.pbData) Then
                Marshal.FreeHGlobal(optEntBlob.pbData)
            End If
        End Try
        Return cipherText
    End Function 'ProtectData
End Class 'DataProtection

